/************************************************************************/
/* ioloop.h								*/
/************************************************************************/
/*
 *  (c)2021,2022 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _IOLOOP_H_
#define _IOLOOP_H_


#include <stdint.h>
#include <sys/epoll.h>
#include <time.h>
#include <dlist.h>



#define IOLOOP_DEFAULT_TIMEOUT		60000

#define POLL_MAX_SOCKETS		16384
#define	EPOLL_TABLE_SIZE		512


#define IOLOOP_POLL_IN			EPOLLIN
#define IOLOOP_POLL_OUT			EPOLLOUT
#define IOLOOP_POLL_ERR			EPOLLERR
#define IOLOOP_POLL_HUP			EPOLLHUP


#define CD_TYPE_COMMAND                 1
#define CD_TYPE_FRONTEND                2
#define CD_TYPE_BACKEND                 3


#define IOLOOP_TIMER_MAX_DEFAULT	16


typedef struct _connection_data_t connection_data_t;
struct _connection_data_t {
	uint8_t		type;
	uint16_t	port;
};


typedef enum {
	IOLOOP_MODIFY_FD_O_EVENTS	= 1,
	_IOLOOP_MODIFY_FD_MAX,
} ioloop_modify_opt_t;


typedef struct _ioloop_modify_arg_t ioloop_modify_arg_t;
struct _ioloop_modify_arg_t {
	union {
		uint32_t	events;
	} u;
};


typedef int (*ioloop_callback_func)(int fd,
				int events,
				connection_data_t *cd,
				void *arg);


typedef struct _ioloop_t ioloop_t;
struct _ioloop_t {
	uint32_t		options;
	uint32_t		flags;
	ioloop_callback_func	default_callback;

	uint32_t		poll_timeout;
//	nfds_t			poll_nfds;
//	struct pollfd		poll_fds[POLL_MAX_SOCKETS];
	connection_data_t	poll_conn_data[POLL_MAX_SOCKETS];

	int			epoll_fd;	// epoll FD
	int			epoll_pipe[2];	// epoll pipe ends

	int32_t			fd_count;
	int			fd_descriptor[POLL_MAX_SOCKETS];
	ioloop_callback_func	fd_callback[POLL_MAX_SOCKETS];
	void			*fd_callback_arg[POLL_MAX_SOCKETS];

	int			timer_max;
	int			timer_nid;
	dlist_t			timer_free_list;
	dlist_t			timer_list;
};


ioloop_t *ioloop_open(int options);
int ioloop_close(ioloop_t *loop);
int ioloop_set_default_io_callback(ioloop_t *loop,
			ioloop_callback_func func);
int ioloop_register_fd(ioloop_t *loop,
			int fd,
			uint8_t type,
			uint16_t port,
			ioloop_callback_func callback,
			void *callback_arg);
int ioloop_unregister_fd(ioloop_t *loop, int fd);
int ioloop_modify_fd(ioloop_t *loop,
			int fd,
			ioloop_modify_opt_t option,
			ioloop_modify_arg_t *arg);
int ioloop_run(ioloop_t *loop, int options, uint32_t timeout);
int ioloop_exit(ioloop_t *loop);



// ioloop timers

// opaque timer object
typedef struct _ioloop_timer_t ioloop_timer_t;

typedef void (*ioloop_timer_callback)(ioloop_timer_t *timer,
				void *arg);


ioloop_timer_t *ioloop_timer_add(ioloop_t *loop,
		int expire_ms,
		ioloop_timer_callback cb,
		void *arg);
//returns:
//  non-NULL a timer handle to be used in subsequent
//           calls for this timer
//  NULL     on error and errno is set appropriately

int ioloop_timer_remove(ioloop_timer_t *timer);

int ioloop_timer_enable(ioloop_timer_t *timer);
int ioloop_timer_disable(ioloop_timer_t *timer);

int ioloop_timer_set_expiration(ioloop_timer_t *timer,
			int expire_ms);
int ioloop_timer_set_arg(ioloop_timer_t *timer,
			void *arg);
int ioloop_timer_set_callback(ioloop_timer_t *timer,
			ioloop_timer_callback cb);



#endif


